@@ -1,8 +1,10 @@
 /*
- * (C) 2005-2008 by Pablo Neira Ayuso <pablo@netfilter.org>
+ * (C) 2005-2011 by Pablo Neira Ayuso <pablo@netfilter.org>
  *
- * This software may be used and distributed according to the terms
- * of the GNU General Public License, incorporated herein by reference.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  */
 
 #ifndef _LIBNETFILTER_CONNTRACK_H_
@@ -147,6 +149,10 @@ enum nf_conntrack_attr_grp {
 	ATTR_GRP_MASTER_PORT,			/* struct nfct_attr_grp_port */
 	ATTR_GRP_ORIG_COUNTERS,			/* struct nfct_attr_grp_ctrs */
 	ATTR_GRP_REPL_COUNTERS,			/* struct nfct_attr_grp_ctrs */
+	ATTR_GRP_ORIG_ADDR_SRC = 12,		/* union nfct_attr_grp_addr */
+	ATTR_GRP_ORIG_ADDR_DST,			/* union nfct_attr_grp_addr */
+	ATTR_GRP_REPL_ADDR_SRC,			/* union nfct_attr_grp_addr */
+	ATTR_GRP_REPL_ADDR_DST,			/* union nfct_attr_grp_addr */
 	ATTR_GRP_MAX
 };
 
@@ -172,6 +178,12 @@ struct nfct_attr_grp_ctrs {
 	u_int64_t bytes;
 };
 
+union nfct_attr_grp_addr {
+	u_int32_t ip;
+	u_int32_t ip6[4];
+	u_int32_t addr[4];
+};
+
 /* message type */
 enum nf_conntrack_msg_type {
 	NFCT_T_UNKNOWN = 0,
@@ -390,6 +402,8 @@ enum nf_conntrack_query {
 	NFCT_Q_DUMP,
 	NFCT_Q_DUMP_RESET,
 	NFCT_Q_CREATE_UPDATE,
+	NFCT_Q_DUMP_FILTER,
+	NFCT_Q_DUMP_FILTER_RESET,
 };
 
 extern int nfct_query(struct nfct_handle *h,
@@ -419,7 +433,7 @@ extern void nfct_copy_attr(struct nf_conntrack *ct1,
 			   const struct nf_conntrack *ct2,
 			   const enum nf_conntrack_attr type);
 
-/* filter */
+/* event filtering */
 
 struct nfct_filter;
 
@@ -470,6 +484,33 @@ extern int nfct_filter_set_logic(struct nfct_filter *filter,
 extern int nfct_filter_attach(int fd, struct nfct_filter *filter);
 extern int nfct_filter_detach(int fd);
 
+/* dump filtering */
+
+struct nfct_filter_dump;
+
+struct nfct_filter_dump_mark {
+	u_int32_t val;
+	u_int32_t mask;
+};
+
+enum nfct_filter_dump_attr {
+	NFCT_FILTER_DUMP_MARK = 0,	/* struct nfct_filter_dump_mark */
+	NFCT_FILTER_DUMP_L3NUM,		/* u_int8_t */
+	NFCT_FILTER_DUMP_MAX
+};
+
+struct nfct_filter_dump *nfct_filter_dump_create(void);
+
+void nfct_filter_dump_destroy(struct nfct_filter_dump *filter);
+
+void nfct_filter_dump_set_attr(struct nfct_filter_dump *filter_dump,
+			       const enum nfct_filter_dump_attr type,
+			       const void *data);
+
+void nfct_filter_dump_set_attr_u8(struct nfct_filter_dump *filter_dump,
+				  const enum nfct_filter_dump_attr type,
+				  u_int8_t data);
+
 /* low level API: netlink functions */
 
 extern __attribute__((deprecated)) int
@@ -507,6 +548,11 @@ enum nf_expect_attr {
 	ATTR_EXP_TIMEOUT,	/* u32 bits */
 	ATTR_EXP_ZONE,		/* u16 bits */
 	ATTR_EXP_FLAGS,		/* u32 bits */
+	ATTR_EXP_HELPER_NAME,	/* string (16 bytes max) */
+	ATTR_EXP_CLASS,		/* u32 bits */
+	ATTR_EXP_NAT_TUPLE,	/* pointer to conntrack object */
+	ATTR_EXP_NAT_DIR,	/* u8 bits */
+	ATTR_EXP_FN,		/* string */
 	ATTR_EXP_MAX
 };
 
@@ -596,6 +642,15 @@ extern int nfexp_snprintf(char *buf,
 			  const unsigned int out_type,
 			  const unsigned int out_flags);
 
+/* compare */
+extern int nfexp_cmp(const struct nf_expect *exp1,
+		     const struct nf_expect *exp2,
+		     unsigned int flags);
+
+extern int nfexp_send(struct nfct_handle *h,
+		      const enum nf_conntrack_query qt,
+		      const void *data);
+
 extern int nfexp_catch(struct nfct_handle *h);
 
 /* low level API */
@@ -710,6 +765,11 @@ enum ip_conntrack_status {
 #define NFCT_DIR_REPLY 1
 #define NFCT_DIR_MAX NFCT_DIR_REPLY+1
 
+/* xt_helper uses a length size of 30 bytes, however, no helper name in
+ * the tree has exceeded 16 bytes length. Since 2.6.29, the maximum
+ * length accepted is 16 bytes, this limit is enforced during module load. */
+#define NFCT_HELPER_NAME_MAX	16
+
 #ifdef __cplusplus
 }
 #endif
