@@ -1,8 +1,10 @@
 /*
- * (C) 2006 by Pablo Neira Ayuso <pablo@netfilter.org>
+ * (C) 2005-2011 by Pablo Neira Ayuso <pablo@netfilter.org>
  *
- * This software may be used and distributed according to the terms
- * of the GNU General Public License, incorporated herein by reference.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  */
 
 #include <stdlib.h>
@@ -375,7 +377,7 @@ void nfct_set_attr(struct nf_conntrack *ct,
 
 	if (set_attr_array[type]) {
 		set_attr_array[type](ct, value);
-		set_bit(type, ct->set);
+		set_bit(type, ct->head.set);
 	}
 }
 
@@ -449,7 +451,7 @@ const void *nfct_get_attr(const struct nf_conntrack *ct,
 		return NULL;
 	}
 
-	if (!test_bit(type, ct->set)) {
+	if (!test_bit(type, ct->head.set)) {
 		errno = ENODATA;
 		return NULL;
 	}
@@ -540,7 +542,7 @@ int nfct_attr_is_set(const struct nf_conntrack *ct,
 		errno = EINVAL;
 		return -1;
 	}
-	return test_bit(type, ct->set);
+	return test_bit(type, ct->head.set);
 }
 
 /**
@@ -565,7 +567,7 @@ int nfct_attr_is_set_array(const struct nf_conntrack *ct,
 			errno = EINVAL;
 			return -1;
 		}
-		if (!test_bit(type_array[i], ct->set))
+		if (!test_bit(type_array[i], ct->head.set))
 			return 0;
 	}
 	return 1;
@@ -588,7 +590,7 @@ int nfct_attr_unset(struct nf_conntrack *ct,
 		errno = EINVAL;
 		return -1;
 	}
-	unset_bit(type, ct->set);
+	unset_bit(type, ct->head.set);
 
 	return 0;
 }
@@ -599,8 +601,8 @@ int nfct_attr_unset(struct nf_conntrack *ct,
  * \param type attribute group (see ATTR_GRP_*)
  * \param data pointer to struct (see struct nfct_attr_grp_*)
  *
- * Note that calling this function for ATTR_GRP_COUNTER_* does nothing since 
- * counters are unsettable.
+ * Note that calling this function for ATTR_GRP_COUNTER_* and ATTR_GRP_ADDR_*
+ * have no effect.
  */
 void nfct_set_attr_grp(struct nf_conntrack *ct,
 		       const enum nf_conntrack_attr_grp type,
@@ -613,7 +615,8 @@ void nfct_set_attr_grp(struct nf_conntrack *ct,
 
 	if (set_attr_grp_array[type]) {
 		set_attr_grp_array[type](ct, data);
-		set_bitmask_u32(ct->set, attr_grp_bitmask[type], __NFCT_BITSET);
+		set_bitmask_u32(ct->head.set,
+				attr_grp_bitmask[type].bitmask, __NFCT_BITSET);
 	}
 }
 
@@ -636,9 +639,23 @@ int nfct_get_attr_grp(const struct nf_conntrack *ct,
 		errno = EINVAL;
 		return -1;
 	}
-	if (!test_bitmask_u32(ct->set, attr_grp_bitmask[type], __NFCT_BITSET)) {
-		errno = ENODATA;
-		return -1;
+	switch(attr_grp_bitmask[type].type) {
+	case NFCT_BITMASK_AND:
+		if (!test_bitmask_u32(ct->head.set,
+				      attr_grp_bitmask[type].bitmask,
+				      __NFCT_BITSET)) {
+			errno = ENODATA;
+			return -1;
+		}
+		break;
+	case NFCT_BITMASK_OR:
+		if (!test_bitmask_u32_or(ct->head.set,
+					 attr_grp_bitmask[type].bitmask,
+					 __NFCT_BITSET)) {
+			errno = ENODATA;
+			return -1;
+		}
+		break;
 	}
 	assert(get_attr_grp_array[type]);
 	get_attr_grp_array[type](ct, data);
@@ -661,7 +678,23 @@ int nfct_attr_grp_is_set(const struct nf_conntrack *ct,
 		errno = EINVAL;
 		return -1;
 	}
-	return test_bitmask_u32(ct->set, attr_grp_bitmask[type], __NFCT_BITSET);
+	switch(attr_grp_bitmask[type].type) {
+	case NFCT_BITMASK_AND:
+		if (test_bitmask_u32(ct->head.set,
+				     attr_grp_bitmask[type].bitmask,
+				     __NFCT_BITSET)) {
+			return 1;
+		}
+		break;
+	case NFCT_BITMASK_OR:
+		if (test_bitmask_u32_or(ct->head.set,
+					attr_grp_bitmask[type].bitmask,
+					__NFCT_BITSET)) {
+			return 1;
+		}
+		break;
+	}
+	return 0;
 }
 
 /**
@@ -681,7 +714,8 @@ int nfct_attr_grp_unset(struct nf_conntrack *ct,
 		errno = EINVAL;
 		return -1;
 	}
-	unset_bitmask_u32(ct->set, attr_grp_bitmask[type], __NFCT_BITSET);
+	unset_bitmask_u32(ct->head.set, attr_grp_bitmask[type].bitmask,
+			  __NFCT_BITSET);
 
 	return 0;
 }
@@ -764,7 +798,14 @@ __build_query_ct(struct nfnl_subsys_handle *ssh,
 	case NFCT_Q_CREATE_UPDATE:
 		__build_conntrack(ssh, req, size, IPCTNL_MSG_CT_NEW, NLM_F_REQUEST|NLM_F_CREATE|NLM_F_ACK, data);
 		break;
-
+	case NFCT_Q_DUMP_FILTER:
+		nfnl_fill_hdr(ssh, &req->nlh, 0, AF_UNSPEC, 0, IPCTNL_MSG_CT_GET, NLM_F_REQUEST|NLM_F_DUMP);
+		__build_filter_dump(req, size, data);
+		break;
+	case NFCT_Q_DUMP_FILTER_RESET:
+		nfnl_fill_hdr(ssh, &req->nlh, 0, AF_UNSPEC, 0, IPCTNL_MSG_CT_GET_CTRZERO, NLM_F_REQUEST|NLM_F_DUMP);
+		__build_filter_dump(req, size, data);
+		break;
 	default:
 		errno = ENOTSUP;
 		return -1;
@@ -800,6 +841,8 @@ __build_query_ct(struct nfnl_subsys_handle *ssh,
  * 	- NFCT_Q_FLUSH: flush the conntrack table
  * 	- NFCT_Q_DUMP: dump the conntrack table
  * 	- NFCT_Q_DUMP_RESET: dump the conntrack table and reset counters
+ * 	- NFCT_Q_DUMP_FILTER: dump the conntrack table
+ * 	- NFCT_Q_DUMP_FILTER_RESET: dump the conntrack table and reset counters
  *
  * Pass a valid pointer to the protocol family (u_int32_t)
  *
@@ -1016,10 +1059,6 @@ int nfct_snprintf(char *buf,
 	return __snprintf_conntrack(buf, size, ct, msg_type, out_type, flags);
 }
 
-/**
- * @}
- */
-
 /**
  * nfct_compare - compare two conntrack objects
  * \param ct1 pointer to a valid conntrack object
@@ -1130,10 +1169,10 @@ void nfct_copy(struct nf_conntrack *ct1,
 	}
 	if (flags == NFCT_CP_ALL) {
 		for (i=0; i<ATTR_MAX; i++) {
-			if (test_bit(i, ct2->set)) {
+			if (test_bit(i, ct2->head.set)) {
 				assert(copy_attr_array[i]);
 				copy_attr_array[i](ct1, ct2);
-				set_bit(i, ct1->set);
+				set_bit(i, ct1->head.set);
 			}
 		}
 		return;
@@ -1156,10 +1195,10 @@ void nfct_copy(struct nf_conntrack *ct1,
 
 	if (flags & NFCT_CP_ORIG) {
 		for (i=0; i<__CP_ORIG_MAX; i++) {
-			if (test_bit(cp_orig_mask[i], ct2->set)) {
+			if (test_bit(cp_orig_mask[i], ct2->head.set)) {
 				assert(copy_attr_array[i]);
 				copy_attr_array[cp_orig_mask[i]](ct1, ct2);
-				set_bit(cp_orig_mask[i], ct1->set);
+				set_bit(cp_orig_mask[i], ct1->head.set);
 			}
 		}
 	}
@@ -1178,20 +1217,20 @@ void nfct_copy(struct nf_conntrack *ct1,
 
 	if (flags & NFCT_CP_REPL) {
 		for (i=0; i<__CP_REPL_MAX; i++) {
-			if (test_bit(cp_repl_mask[i], ct2->set)) {
+			if (test_bit(cp_repl_mask[i], ct2->head.set)) {
 				assert(copy_attr_array[i]);
 				copy_attr_array[cp_repl_mask[i]](ct1, ct2);
-				set_bit(cp_repl_mask[i], ct1->set);
+				set_bit(cp_repl_mask[i], ct1->head.set);
 			}
 		}
 	}
 
 	if (flags & NFCT_CP_META) {
 		for (i=ATTR_TCP_STATE; i<ATTR_MAX; i++) {
-			if (test_bit(i, ct2->set)) {
+			if (test_bit(i, ct2->head.set)) {
 				assert(copy_attr_array[i]),
 				copy_attr_array[i](ct1, ct2);
-				set_bit(i, ct1->set);
+				set_bit(i, ct1->head.set);
 			}
 		}
 	}
@@ -1209,10 +1248,10 @@ void nfct_copy_attr(struct nf_conntrack *ct1,
 		    const struct nf_conntrack *ct2,
 		    const enum nf_conntrack_attr type)
 {
-	if (test_bit(type, ct2->set)) {
+	if (test_bit(type, ct2->head.set)) {
 		assert(copy_attr_array[type]);
 		copy_attr_array[type](ct1, ct2);
-		set_bit(type, ct1->set);
+		set_bit(type, ct1->head.set);
 	}
 }
 
@@ -1358,3 +1397,72 @@ int nfct_filter_detach(int fd)
 /**
  * @}
  */
+
+/**
+ * \defgroup dumpfilter Kernel-space filtering for dumping
+ *
+ * @{
+ */
+
+/**
+ * nfct_filter_dump_create - create a dump filter
+ *
+ * This function returns a valid pointer on success, otherwise NULL is
+ * returned and errno is appropriately set.
+ */
+struct nfct_filter_dump *nfct_filter_dump_create(void)
+{
+	return calloc(sizeof(struct nfct_filter_dump), 1);
+}
+
+/**
+ * nfct_filter_dump_destroy - destroy a dump filter
+ * \param filter filter that we want to destroy
+ *
+ * This function releases the memory that is used by the filter object.
+ */
+void nfct_filter_dump_destroy(struct nfct_filter_dump *filter)
+{
+	assert(filter != NULL);
+	free(filter);
+	filter = NULL;
+}
+
+/**
+ * nfct_filter_dump_attr_set - set filter attribute
+ * \param filter dump filter object that we want to modify
+ * \param type filter attribute type
+ * \param value pointer to the value of the filter attribute
+ */
+void nfct_filter_dump_set_attr(struct nfct_filter_dump *filter_dump,
+			       const enum nfct_filter_dump_attr type,
+			       const void *value)
+{
+	assert(filter_dump != NULL);
+	assert(value != NULL);
+
+	if (unlikely(type >= NFCT_FILTER_DUMP_MAX))
+		return;
+
+	if (set_filter_dump_attr_array[type]) {
+		set_filter_dump_attr_array[type](filter_dump, value);
+		filter_dump->set |= (1 << type);
+	}
+}
+
+/**
+ * nfct_filter_dump_attr_set_u8 - set u8 dump filter attribute
+ * \param filter dump filter object that we want to modify
+ * \param type filter attribute type
+ * \param value value of the filter attribute using unsigned int (32 bits).
+ */
+void nfct_filter_dump_set_attr_u8(struct nfct_filter_dump *filter_dump,
+				  const enum nfct_filter_dump_attr type,
+				  u_int8_t value)
+{
+	nfct_filter_dump_set_attr(filter_dump, type, &value);
+}
+
+/**
+ * @}
+ */
