@@ -1,8 +1,10 @@
 /*
- * (C) 2006 by Pablo Neira Ayuso <pablo@netfilter.org>
+ * (C) 2005-2011 by Pablo Neira Ayuso <pablo@netfilter.org>
  *
- * This software may be used and distributed according to the terms
- * of the GNU General Public License, incorporated herein by reference.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  */
 
 #include "internal/internal.h"
@@ -51,17 +53,12 @@
  * </flow>
  */
 
-enum {
-	__ADDR_SRC = 0,
-	__ADDR_DST,
-};
-
-static const char *__proto2str(u_int8_t protonum)
+const char *__proto2str(u_int8_t protonum)
 {
 	return proto2str[protonum] ? proto2str[protonum] : "unknown";
 }
 
-static const char *__l3proto2str(u_int8_t protonum)
+const char *__l3proto2str(u_int8_t protonum)
 {
 	return l3proto2str[protonum] ? l3proto2str[protonum] : "unknown";
 }
@@ -95,24 +92,16 @@ static int __snprintf_ipv6_xml(char *buf,
 	return snprintf(buf, len, "%s", tmp);
 }
 
-static int __snprintf_addr_xml(char *buf,
-			       unsigned int len,
-			       const struct __nfct_tuple *tuple, 
-			       unsigned int type)
+int __snprintf_addr_xml(char *buf, unsigned int len,
+			const struct __nfct_tuple *tuple,
+			enum __nfct_addr type)
 {
 	int ret;
 	unsigned int size = 0, offset = 0;
+	const char *tag = (type == __ADDR_SRC) ? "src" : "dst";
 
-	switch(type) {
-	case __ADDR_SRC:
-		ret = snprintf(buf, len, "<src>");
-		BUFFER_SIZE(ret, size, len, offset);
-		break;
-	case __ADDR_DST:
-		ret = snprintf(buf+offset, len, "<dst>");
-		BUFFER_SIZE(ret, size, len, offset);
-		break;
-	}
+	ret = snprintf(buf, len, "<%s>", tag);
+	BUFFER_SIZE(ret, size, len, offset);
 
 	switch (tuple->l3protonum) {
 	case AF_INET:
@@ -125,24 +114,15 @@ static int __snprintf_addr_xml(char *buf,
 		break;
 	}
 
-	switch(type) {
-	case __ADDR_SRC:
-		ret = snprintf(buf+offset, len, "</src>");
-		BUFFER_SIZE(ret, size, len, offset);
-		break;
-	case __ADDR_DST:
-		ret = snprintf(buf+offset, len, "</dst>");
-		BUFFER_SIZE(ret, size, len, offset);
-		break;
-	}
+	ret = snprintf(buf+offset, len, "</%s>", tag);
+	BUFFER_SIZE(ret, size, len, offset);
 
 	return size;
 }
 
-static int __snprintf_proto_xml(char *buf,
-				unsigned int len,
-				const struct __nfct_tuple *tuple, 
-				unsigned int type)
+int __snprintf_proto_xml(char *buf, unsigned int len,
+			 const struct __nfct_tuple *tuple,
+			 enum __nfct_addr type)
 {
 	int ret = 0;
 	unsigned int size = 0, offset = 0;
@@ -198,6 +178,97 @@ static int __snprintf_counters_xml(char *buf,
 	return size;
 }
 
+static int
+__snprintf_timestamp_start(char *buf, unsigned int len,
+			   const struct nf_conntrack *ct)
+{
+	int ret;
+	unsigned int size = 0, offset = 0;
+
+	ret = snprintf(buf, len, "<start>%llu</start>",
+		       (unsigned long long)ct->timestamp.start);
+	BUFFER_SIZE(ret, size, len, offset);
+
+	return size;
+}
+
+static int
+__snprintf_timestamp_stop(char *buf, unsigned int len,
+			  const struct nf_conntrack *ct)
+{
+	int ret;
+	unsigned int size = 0, offset = 0;
+
+	ret = snprintf(buf, len, "<stop>%llu</stop>",
+		       (unsigned long long)ct->timestamp.stop);
+	BUFFER_SIZE(ret, size, len, offset);
+
+	return size;
+}
+
+static int
+__snprintf_deltatime_now(char *buf, unsigned int len,
+			 const struct nf_conntrack *ct)
+{
+	int ret;
+	unsigned int size = 0, offset = 0;
+	time_t now, delta_time;
+
+	time(&now);
+        delta_time = now - (time_t)(ct->timestamp.start / NSEC_PER_SEC);
+
+	ret = snprintf(buf+offset, len, "<deltatime>%llu</deltatime>",
+		(unsigned long long)delta_time);
+	BUFFER_SIZE(ret, size, len, offset);
+
+	return size;
+}
+
+static int
+__snprintf_deltatime(char *buf, unsigned int len, const struct nf_conntrack *ct)
+{
+	int ret;
+	unsigned int size = 0, offset = 0;
+	time_t delta_time = (time_t)((ct->timestamp.stop -
+				ct->timestamp.start) / NSEC_PER_SEC);
+
+	ret = snprintf(buf+offset, len, "<deltatime>%llu</deltatime>",
+		(unsigned long long)delta_time);
+	BUFFER_SIZE(ret, size, len, offset);
+
+	return size;
+}
+
+int
+__snprintf_localtime_xml(char *buf, unsigned int len, const struct tm *tm)
+{
+	int ret = 0;
+	unsigned int size = 0, offset = 0;
+
+	ret = snprintf(buf+offset, len, "<hour>%d</hour>", tm->tm_hour);
+	BUFFER_SIZE(ret, size, len, offset);
+
+	ret = snprintf(buf+offset, len, "<min>%02d</min>", tm->tm_min);
+	BUFFER_SIZE(ret, size, len, offset);
+
+	ret = snprintf(buf+offset, len, "<sec>%02d</sec>", tm->tm_sec);
+	BUFFER_SIZE(ret, size, len, offset);
+
+	ret = snprintf(buf+offset, len, "<wday>%d</wday>", tm->tm_wday + 1);
+	BUFFER_SIZE(ret, size, len, offset);
+
+	ret = snprintf(buf+offset, len, "<day>%d</day>", tm->tm_mday);
+	BUFFER_SIZE(ret, size, len, offset);
+
+	ret = snprintf(buf+offset, len, "<month>%d</month>", tm->tm_mon + 1);
+	BUFFER_SIZE(ret, size, len, offset);
+
+	ret = snprintf(buf+offset, len, "<year>%d</year>", 1900 + tm->tm_year);
+	BUFFER_SIZE(ret, size, len, offset);
+
+	return size;
+}
+
 static int __snprintf_tuple_xml(char *buf,
 				unsigned int len,
 				const struct nf_conntrack *ct,
@@ -205,8 +276,16 @@ static int __snprintf_tuple_xml(char *buf,
 {
 	int ret;
 	unsigned int size = 0, offset = 0;
-	const struct __nfct_tuple *tuple = &ct->tuple[dir];
+	const struct __nfct_tuple *tuple = NULL;
 
+	switch(dir) {
+	case __DIR_ORIG:
+		tuple = &ct->head.orig;
+		break;
+	case __DIR_REPL:
+		tuple = &ct->repl;
+		break;
+	}
 	ret = snprintf(buf, len, "<meta direction=\"%s\">",
 		       dir == __DIR_ORIG ? "original" : "reply");
 	BUFFER_SIZE(ret, size, len, offset);
@@ -216,10 +295,10 @@ static int __snprintf_tuple_xml(char *buf,
 		       tuple->l3protonum, __l3proto2str(tuple->l3protonum));
 	BUFFER_SIZE(ret, size, len, offset);
 
-	ret = __snprintf_addr_xml(buf+offset, len, tuple, __DIR_ORIG);
+	ret = __snprintf_addr_xml(buf+offset, len, tuple, __ADDR_SRC);
 	BUFFER_SIZE(ret, size, len, offset);
 
-	ret = __snprintf_addr_xml(buf+offset, len, tuple, __DIR_REPL);
+	ret = __snprintf_addr_xml(buf+offset, len, tuple, __ADDR_DST);
 	BUFFER_SIZE(ret, size, len, offset);
 
 	ret = snprintf(buf+offset, len, "</layer3>");
@@ -239,8 +318,8 @@ static int __snprintf_tuple_xml(char *buf,
 	ret = snprintf(buf+offset, len, "</layer4>");
 	BUFFER_SIZE(ret, size, len, offset);
 
-	if (test_bit(ATTR_ORIG_COUNTER_PACKETS, ct->set) &&
-	    test_bit(ATTR_ORIG_COUNTER_BYTES, ct->set)) {
+	if (test_bit(ATTR_ORIG_COUNTER_PACKETS, ct->head.set) &&
+	    test_bit(ATTR_ORIG_COUNTER_BYTES, ct->head.set)) {
 		ret = snprintf(buf+offset, len, "<counters>");
 		BUFFER_SIZE(ret, size, len, offset);
 
@@ -289,19 +368,24 @@ int __snprintf_conntrack_xml(char *buf,
 	ret = __snprintf_tuple_xml(buf+offset, len, ct, __DIR_REPL);
 	BUFFER_SIZE(ret, size, len, offset);
 
-	if (test_bit(ATTR_TCP_STATE, ct->set) ||
-	    test_bit(ATTR_SCTP_STATE, ct->set) ||
-	    test_bit(ATTR_DCCP_STATE, ct->set) ||
-	    test_bit(ATTR_TIMEOUT, ct->set) ||
-	    test_bit(ATTR_MARK, ct->set) ||
-	    test_bit(ATTR_USE, ct->set) ||
-	    test_bit(ATTR_STATUS, ct->set)) {
+	if (test_bit(ATTR_TCP_STATE, ct->head.set) ||
+	    test_bit(ATTR_SCTP_STATE, ct->head.set) ||
+	    test_bit(ATTR_DCCP_STATE, ct->head.set) ||
+	    test_bit(ATTR_TIMEOUT, ct->head.set) ||
+	    test_bit(ATTR_MARK, ct->head.set) ||
+	    test_bit(ATTR_SECMARK, ct->head.set) ||
+	    test_bit(ATTR_ZONE, ct->head.set) ||
+	    test_bit(ATTR_USE, ct->head.set) ||
+	    test_bit(ATTR_STATUS, ct->head.set) ||
+	    test_bit(ATTR_ID, ct->head.set) ||
+	    test_bit(ATTR_TIMESTAMP_START, ct->head.set) ||
+	    test_bit(ATTR_TIMESTAMP_STOP, ct->head.set)) {
 		ret = snprintf(buf+offset, len, 
 			       "<meta direction=\"independent\">");
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
-	if (test_bit(ATTR_TCP_STATE, ct->set)) {
+	if (test_bit(ATTR_TCP_STATE, ct->head.set)) {
 		ret = snprintf(buf+offset, len, "<state>%s</state>",
 			       ct->protoinfo.tcp.state < TCP_CONNTRACK_MAX ?
 			       states[ct->protoinfo.tcp.state] :
@@ -309,7 +393,7 @@ int __snprintf_conntrack_xml(char *buf,
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
-	if (test_bit(ATTR_SCTP_STATE, ct->set)) {
+	if (test_bit(ATTR_SCTP_STATE, ct->head.set)) {
 		ret = snprintf(buf+offset, len, "<state>%s</state>",
 			       ct->protoinfo.sctp.state < SCTP_CONNTRACK_MAX ?
 			       states[ct->protoinfo.sctp.state] :
@@ -317,7 +401,7 @@ int __snprintf_conntrack_xml(char *buf,
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
-	if (test_bit(ATTR_DCCP_STATE, ct->set)) {
+	if (test_bit(ATTR_DCCP_STATE, ct->head.set)) {
 		ret = snprintf(buf+offset, len, "<state>%s</state>",
 			       ct->protoinfo.sctp.state < DCCP_CONNTRACK_MAX ?
 			       states[ct->protoinfo.dccp.state] :
@@ -325,52 +409,97 @@ int __snprintf_conntrack_xml(char *buf,
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
-	if (test_bit(ATTR_TIMEOUT, ct->set)) {
+	if (test_bit(ATTR_TIMEOUT, ct->head.set)) {
 		ret = snprintf(buf+offset, len,
 				"<timeout>%u</timeout>", ct->timeout);
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
-	if (test_bit(ATTR_MARK, ct->set)) {
+	if (test_bit(ATTR_MARK, ct->head.set)) {
 		ret = snprintf(buf+offset, len, "<mark>%u</mark>", ct->mark);
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
-	if (test_bit(ATTR_SECMARK, ct->set)) {
+	if (test_bit(ATTR_SECMARK, ct->head.set)) {
 		ret = snprintf(buf+offset, len, 
 				"<secmark>%u</secmark>", ct->secmark);
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
-	if (test_bit(ATTR_USE, ct->set)) {
+	if (test_bit(ATTR_SECCTX, ct->head.set)) {
+		ret = snprintf(buf+offset, len,
+				"<secctx>%s</secctx>", ct->secctx);
+		BUFFER_SIZE(ret, size, len, offset);
+	}
+
+	if (test_bit(ATTR_ZONE, ct->head.set)) {
+		ret = snprintf(buf+offset, len, "<zone>%u</zone>", ct->zone);
+		BUFFER_SIZE(ret, size, len, offset);
+	}
+
+	if (test_bit(ATTR_USE, ct->head.set)) {
 		ret = snprintf(buf+offset, len, "<use>%u</use>", ct->use);
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
-	if (test_bit(ATTR_ID, ct->set)) {
+	if (test_bit(ATTR_ID, ct->head.set)) {
 		ret = snprintf(buf+offset, len, "<id>%u</id>", ct->id);
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
-	if (test_bit(ATTR_STATUS, ct->set)
+	if (test_bit(ATTR_STATUS, ct->head.set)
 	    && ct->status & IPS_ASSURED) {
 		ret = snprintf(buf+offset, len, "<assured/>");
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
-	if (test_bit(ATTR_STATUS, ct->set) 
+	if (test_bit(ATTR_STATUS, ct->head.set) 
 	    && !(ct->status & IPS_SEEN_REPLY)) {
 		ret = snprintf(buf+offset, len, "<unreplied/>");
 		BUFFER_SIZE(ret, size, len, offset);
 	}
 
-	if (test_bit(ATTR_TCP_STATE, ct->set) ||
-	    test_bit(ATTR_SCTP_STATE, ct->set) ||
-	    test_bit(ATTR_DCCP_STATE, ct->set) ||
-	    test_bit(ATTR_TIMEOUT, ct->set) ||
-	    test_bit(ATTR_MARK, ct->set) ||
-	    test_bit(ATTR_USE, ct->set) ||
-	    test_bit(ATTR_STATUS, ct->set)) {
+	if (flags & NFCT_OF_TIMESTAMP) {
+		if (test_bit(ATTR_TIMESTAMP_START, ct->head.set) ||
+		    test_bit(ATTR_TIMESTAMP_STOP, ct->head.set)) {
+			ret = snprintf(buf+offset, len, "<timestamp>");
+			BUFFER_SIZE(ret, size, len, offset);
+		}
+		if (test_bit(ATTR_TIMESTAMP_START, ct->head.set)) {
+	 		ret = __snprintf_timestamp_start(buf+offset, len, ct);
+			BUFFER_SIZE(ret, size, len, offset);
+		}
+		if (test_bit(ATTR_TIMESTAMP_STOP, ct->head.set)) {
+	 		ret = __snprintf_timestamp_stop(buf+offset, len, ct);
+			BUFFER_SIZE(ret, size, len, offset);
+		}
+		if (test_bit(ATTR_TIMESTAMP_START, ct->head.set) ||
+		    test_bit(ATTR_TIMESTAMP_STOP, ct->head.set)) {
+			ret = snprintf(buf+offset, len, "</timestamp>");
+			BUFFER_SIZE(ret, size, len, offset);
+		}
+	}
+	if (test_bit(ATTR_TIMESTAMP_START, ct->head.set) &&
+	    test_bit(ATTR_TIMESTAMP_STOP, ct->head.set)) {
+		ret = __snprintf_deltatime(buf+offset, len, ct);
+		BUFFER_SIZE(ret, size, len, offset);
+	} else if (test_bit(ATTR_TIMESTAMP_START, ct->head.set)) {
+		ret = __snprintf_deltatime_now(buf+offset, len, ct);
+		BUFFER_SIZE(ret, size, len, offset);
+	}
+
+	if (test_bit(ATTR_TCP_STATE, ct->head.set) ||
+	    test_bit(ATTR_SCTP_STATE, ct->head.set) ||
+	    test_bit(ATTR_DCCP_STATE, ct->head.set) ||
+	    test_bit(ATTR_TIMEOUT, ct->head.set) ||
+	    test_bit(ATTR_MARK, ct->head.set) ||
+	    test_bit(ATTR_SECMARK, ct->head.set) ||
+	    test_bit(ATTR_ZONE, ct->head.set) ||
+	    test_bit(ATTR_USE, ct->head.set) ||
+	    test_bit(ATTR_STATUS, ct->head.set) ||
+	    test_bit(ATTR_ID, ct->head.set) ||
+	    test_bit(ATTR_TIMESTAMP_START, ct->head.set) ||
+	    test_bit(ATTR_TIMESTAMP_STOP, ct->head.set)) {
 	    	ret = snprintf(buf+offset, len, "</meta>");
 		BUFFER_SIZE(ret, size, len, offset);
 	}
@@ -386,28 +515,7 @@ int __snprintf_conntrack_xml(char *buf,
 		ret = snprintf(buf+offset, len, "<when>");
 		BUFFER_SIZE(ret, size, len, offset);
 
-		ret = snprintf(buf+offset, len, "<hour>%d</hour>", tm.tm_hour);
-		BUFFER_SIZE(ret, size, len, offset);
-
-		ret = snprintf(buf+offset, len, "<min>%02d</min>", tm.tm_min);
-		BUFFER_SIZE(ret, size, len, offset);
-
-		ret = snprintf(buf+offset, len, "<sec>%02d</sec>", tm.tm_sec);
-		BUFFER_SIZE(ret, size, len, offset);
-
-		ret = snprintf(buf+offset, len, "<wday>%d</wday>", 
-			       tm.tm_wday + 1);
-		BUFFER_SIZE(ret, size, len, offset);
-
-		ret = snprintf(buf+offset, len, "<day>%d</day>", tm.tm_mday);
-		BUFFER_SIZE(ret, size, len, offset);
-
-		ret = snprintf(buf+offset, len, "<month>%d</month>", 
-			       tm.tm_mon + 1);
-		BUFFER_SIZE(ret, size, len, offset);
-
-		ret = snprintf(buf+offset, len, "<year>%d</year>", 
-			       1900 + tm.tm_year);
+		ret = __snprintf_localtime_xml(buf+offset, len, &tm);
 		BUFFER_SIZE(ret, size, len, offset);
 
 		ret = snprintf(buf+offset, len, "</when>");
